@@ -375,7 +375,8 @@ extern VerboseMode verbose_mode;
  */
 enum ConsensusType {
     CT_NONE, CT_CONSENSUS_TREE, CT_CONSENSUS_NETWORK,
-    CT_ASSIGN_SUPPORT, CT_ASSIGN_SUPPORT_EXTENDED, COMPARE
+    CT_ASSIGN_SUPPORT, CT_ASSIGN_SUPPORT_EXTENDED, COMPARE,
+    CT_ROOTSTRAP
 };
 
 enum TestType {
